import os
import sys
from sqlalchemy import Column, ForeignKey, Integer, String, DateTime, Boolean
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.orm import Session
import yaml
from sqlalchemy import select
from flask_login import UserMixin


def get_connection_str():
    return os.getenv('CX_DB_CONN')


Base = declarative_base()


class UserObj(UserMixin):
    def __init__(self, id_, name, email):
        self.id = id_
        self.name = name
        self.email = email


class User(Base):
    __tablename__ = 'user'
    id = Column(Integer, primary_key=True)
    google_id = Column(String(64), nullable=True, unique=False) 
    username = Column(String(50), nullable=False, unique=True)
    password = Column(String(100), nullable=False)
    email = Column(String(50), nullable=False)
    is_active = Column(Boolean, nullable=False, default=True) 
    private_key = Column(String(4000), nullable=True)
    public_key = Column(String(1000), nullable=True)


class IDE(Base):
    __tablename__ = 'ide'
    id = Column(Integer, primary_key=True)
    name = Column(String(30))
 

class UserRepo(Base):
    __tablename__ = 'user_repo'
    id = Column(Integer, primary_key=True)
    uri = Column(String(100))
    user_id = Column(Integer, ForeignKey('user.id'))
    user = relationship(User)


class RuntimeInstall(Base):
    __tablename__ = 'runtime_install'
    id = Column(Integer, primary_key=True)
    name = Column(String(30), unique=True)
    script_body = Column(String(4000))


class UserIDE(Base):
    __tablename__ = 'user_ide'
    id = Column(Integer, primary_key=True)
    user_id = Column(Integer, ForeignKey('user.id'))
    ide_id = Column(Integer, ForeignKey('ide.id'))
    user = relationship(User)
    ide = relationship(IDE)


class IDERuntimeInstall(Base):
    __tablename__ = 'ide_runtime_install'
    id = Column(Integer, primary_key=True)
    user_ide_id = Column(Integer, ForeignKey('user_ide.id'))
    runtime_install_id = Column(Integer, ForeignKey('runtime_install.id'))
    user_ide = relationship(UserIDE)
    runtime_install = relationship(RuntimeInstall)


class IDERepo(Base):
    __tablename__ = 'ide_repo'
    id = Column(Integer, primary_key=True)
    user_ide_id = Column(Integer, ForeignKey('user_ide.id'))
    user_ide = relationship(UserIDE)
    uri = Column(String(100))


class Enrollment(Base):
    __tablename__ = 'enrollment'
    id = Column(Integer, primary_key=True)
    hashed_email = Column(String(64), nullable=False, unique=True)

    
def query_data():
    engine = create_engine(get_connection_str())
    Base.metadata.bind = engine    
    DBSession = sessionmaker(bind=engine)
    session = DBSession()    
    usernames = session.query(UserIDE).all()
    for username in usernames:
        print(f'{username.ide.name}-{username.user.username}')

    user_repos = session.query(UserRepo).all()
    for user_repo in user_repos:
        print(f'{user_repo.uri}')

    user_runtime_installs = session.query(IDERuntimeInstall).all()
    for user_runtime_install in user_runtime_installs:
        print(f'{user_runtime_install.runtime_install.name}')


def drop_create_schema(): 
    engine = create_engine(get_connection_str())
    Base.metadata.bind = engine
    Base.metadata.drop_all()
    Base.metadata.create_all(engine)


if __name__ == '__main__':
    drop_create_schema()


import os
import sys
import shlex
import click
import requests
from os.path import expanduser

import sqlalchemy
from vag.utils import exec, user_util
from vag.utils import config
from vag.utils.misc import create_ssh
from vag.utils.cx_schema import *
from vag.utils.cx_db_util import *
from vag.utils.cx_test_data import *
import vag.utils.gitea_api_util as gitutil
import yaml
from psycopg2.errors import *
from sqlalchemy.exc import IntegrityError, NoResultFound


@click.group()
def cx():
    """ CX automation """
    pass


@cx.command()
@click.argument('username', metavar='<username>')
@click.argument('password', metavar='<password>')
@click.argument('email', metavar='<email>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def add_user(username: str, password: str, email: str, debug: bool):
    """Adds user"""

    user_util.add_user(username, password, email, '')


@cx.command()
@click.argument('username', metavar='<username>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def delete_user(username: str, debug: bool):
    """Clones user"""

    try:
        user = find_user_by_username(username)
    except NoResultFound:
        print(f'there is no user with name {username}')
        sys.exit(1)

    session = get_session()

    user_ides = find_user_ides_by_user_id(user.id)
    for ui in user_ides:

        ide_runtime_installs = find_ide_runtime_installs_by_user_id(ui.id)
        for i in ide_runtime_installs:
            print(f'deleting ide_runtime_install : {i.runtime_install.name}')
            session.delete(i)

        ide_repos = find_ide_repos_by_user_ide_id(ui.id)        
        for ir in ide_repos:
            print(f'deleting ide_repo            : {ir.uri}')
            session.delete(ir)

        print(f'deleting user_ide            : {ui.ide.name}')
        session.delete(ui)

    user_repos = find_user_repos_by_user_id(user.id)    
    for user_repo in user_repos:
        print(f'deleting user_repo           : {user_repo.uri}')
        session.delete(user_repo)

    session = get_session()
    print(f'deleting user                : {user.username}')
    session.delete(user)
    session.commit()


@cx.command()
@click.argument('username', metavar='<username>')
@click.argument('repo', metavar='<repo>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def add_user_repo(username: str, repo: str, debug: bool):
    """add repo to user's list of repos"""

    user = find_user_by_username(username)
    session = get_session()
    user_repo = UserRepo(uri=repo, user_id=user.id)
    session.add(user_repo)
    session.commit()


@cx.command()
@click.argument('name', metavar='<name>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def add_runtime_install(name: str, debug: bool):
    """add runtime install to user's list of runtime installs"""

    document = ""
    for line in sys.stdin:
        document += line

    runtime_install = RuntimeInstall(name=name, script_body=document)
    session = get_session()
    session.add(runtime_install)
    session.commit()


@cx.command()
@click.argument('username', metavar='<username>')
@click.argument('ide_name', metavar='<ide_name>')
@click.argument('runtime_install_name', metavar='<runtime_install_name>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def add_ide_runtime_install(username: str, ide_name: str, runtime_install_name: str, debug: bool):
    """add runtime install to user's list of runtime installs"""

    user = find_user_by_username(username)
    user_ide = find_user_ide_by_user_id_ide_name(user.id, ide_name)
    runtime_install = find_runtime_install_by_name(runtime_install_name)

    session = get_session()
    user_runtime_install = IDERuntimeInstall(user_ide_id=user_ide.id, runtime_install_id=runtime_install.id)
    session.add(user_runtime_install)
    session.commit()


@cx.command()
@click.argument('username', metavar='<username>')
@click.argument('ide_name', metavar='<ide_name>')
@click.argument('repo', metavar='<repo>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def add_ide_repo(username: str, ide_name: str, repo: str, debug: bool):
    """add repo to user's list of repos"""

    user = find_user_by_username(username)
    session = get_session()
    user_ide = find_user_ide_by_user_id_ide_name(user.id, ide_name)
    ide_repo = IDERepo(uri=repo, user_ide_id=user_ide.id)
    session.add(ide_repo)
    session.commit()


@cx.command()
@click.argument('username', metavar='<username>')
@click.argument('ide_name', metavar='<ide_name>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def add_user_ide(username: str, ide_name: str, debug: bool):
    """add runtime install to user's list of runtime installs"""

    user = find_user_by_username(username)
    ide = find_ide_by_name(ide_name)
    session = get_session()
    user_ide = UserIDE(user_id=user.id, ide_id=ide.id)
    session.add(user_ide)
    session.commit()


@cx.command()
@click.argument('username', metavar='<username>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def user_private_key(username: str, debug: bool):
    """update user private key"""

    document = ""
    for line in sys.stdin:
        document += line

    user = find_user_by_username(username)
    user.private_key = document
    session = get_session()
    session.add(user)
    session.commit()


@cx.command()
@click.argument('username', metavar='<username>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def user_public_key(username: str, debug: bool):
    """update user public key"""

    document = ""
    for line in sys.stdin:
        document += line

    user = find_user_by_username(username)
    user.public_key = document
    session = get_session()
    session.add(user)
    session.commit()


yaml.SafeDumper.org_represent_str = yaml.SafeDumper.represent_str

def repr_str(dumper, data):
    if '\n' in data:
        return dumper.represent_scalar(u'tag:yaml.org,2002:str', data, style='|')
    return dumper.org_represent_str(data)

yaml.add_representer(str, repr_str, Dumper=yaml.SafeDumper)


@cx.command()
@click.argument('username', metavar='<username>')
@click.argument('ide', metavar='<ide>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def get_profile(username: str, ide: str, debug: bool):
    """Prints user ide build profile"""

    profile = get_build_profile(username, ide)
    print(yaml.safe_dump(profile))


@cx.command()
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def reset_test_data(debug: bool):
    """Resets test data"""

    print('resetting test data')
    reset_test_data_foo()


@cx.command()
@click.argument('email', metavar='<email>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def add_enrollment(email: str, debug: bool):
    """Adds enrollment"""

    user_util.add_enrollment(email)


@cx.command()
@click.argument('email', metavar='<email>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def remove_enrollment(email: str, debug: bool):
    """Removes enrollment"""

    session = get_session()
    enrollment = find_enrollment_by_hashed_email(hashed_email=user_util.hashed(email))
    if enrollment:
        session.delete(enrollment)
    session.commit()

 


import os
import sys
import shlex
import click
import requests
from os.path import expanduser
from vag.utils import exec
from vag.utils import config
from vag.utils.misc import create_ssh
from vag.utils.cx_schema import *
from vag.utils.cx_db_util import *
from vag.utils.cx_test_data import *
import vag.utils.gitea_api_util as gitutil
import yaml
from sqlalchemy.exc import NoResultFound

@click.group()
def gitea():
    """ Gitea automation """
    pass


@gitea.command()
@click.argument('username', metavar='<username>')
@click.argument('password', metavar='<password>')
@click.argument('email', metavar='<email>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def add_user(username: str, password: str, email: str, debug: bool):
    """Adds git user"""
    
    gitutil.create_user(username, password, email)


@gitea.command()
@click.argument('username', metavar='<username>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def delete_user(username: str, debug: bool):
    """Deletes git user"""

    gitutil.delete_user(username)


@gitea.command()
@click.argument('username', metavar='<username>')
@click.argument('repo_name', metavar='<repo_name>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def create_user_repo(username: str, repo_name: str, debug: bool):
    """Creates git user repo"""
    gitutil.create_user_repo(username, repo_name)


@gitea.command()
@click.argument('username', metavar='<username>')
@click.argument('repo_name', metavar='<repo_name>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def delete_user_repo(username: str, repo_name: str, debug: bool):
    """Deletes git user repo"""
    try:
        user = find_user_by_username(username)
    except NoResultFound:
        print(f'user {username} does not exist')
        sys.exit(1)
    
    gitutil.delete_user_repo(username, repo_name, user.password, debug)


@gitea.command()
@click.argument('username', metavar='<username>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def list_user_repos(username: str, debug: bool):
    """Deletes git user repo"""
    
    response = gitutil.list_user_repos(username)
    for repo in response:
        print(repo.name)


@gitea.command()
@click.argument('username', metavar='<username>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def create_public_key(username: str, debug: bool):
    """Creates git user public key"""
    
    document = ""
    for line in sys.stdin:
        document += line

    gitutil.create_public_key(username, document)




from re import I
from vag.utils.cx_schema import *
from vag.utils.cx_db_util import *
from sqlalchemy.exc import IntegrityError, NoResultFound
import hashlib


def add_user(username: str, password: str, email: str, google_id: str, exitOnFailure=True) -> User:
    session = get_session()
    username = username.lower()
    new_user = User(username=username, password=password, email=email, google_id=google_id)
    session.add(new_user)
    try:
        session.commit()
    except IntegrityError:
        print(f'user {username} already exists')
        if exitOnFailure:
            sys.exit(1)
        else:            
            return None

    print("adding user_repo")
    user_repo = UserRepo(uri=f'ssh://gitea@git-ssh.curiosityworks.org:2222/{username}/project.git ', user=new_user)
    session.add(user_repo)
    session.commit()

    print("adding user_ide")
    ide = find_ide_by_name('vscode')
    user_ide = UserIDE(user=new_user, ide=ide)
    session.add(user_ide)                          
    session.commit()

    print("adding user_runtime_install")
    runtime_install = find_runtime_install_by_name('tmux')
    user_runtime_install = IDERuntimeInstall(user_ide_id=user_ide.id, runtime_install_id=runtime_install.id)
    session.add(user_runtime_install)
    session.commit()

    return new_user


def add_enrollment(email: str, exitOnFailure=True) -> Enrollment:
    session = get_session()
    hashed_email = hashed(email)
    new_enrollment = Enrollment(hashed_email=hashed_email)
    session.add(new_enrollment)
    try:
        session.commit()
    except IntegrityError:
        print(f'enrollment for {email} already exists')
        if exitOnFailure:
            sys.exit(1)
        else:            
            return None


def hashed(s: str) -> str:
    return hashlib.sha256(bytes(s, 'utf-8')).hexdigest()            



from __future__ import print_function
import time
import giteapy
from giteapy.rest import ApiException
from pprint import pprint
import os
import sys

from sqlalchemy.sql.functions import user


def get_gitea_access_token():
    access_token = os.getenv('GITEA_ACCESS_TOKEN')
    if not access_token:
        print('GITEA_ACCESS_TOKEN is missing')
        sys.exit(1)
    return access_token


def get_api_instance() -> giteapy.AdminApi:
    configuration = giteapy.Configuration()
    configuration.api_key['access_token'] = get_gitea_access_token()
    configuration.host = 'https://git.curiosityworks.org/api/v1'
    return giteapy.AdminApi(giteapy.ApiClient(configuration))


def get_user_api_instance() -> giteapy.UserApi:
    configuration = giteapy.Configuration()
    configuration.api_key['access_token'] = get_gitea_access_token()
    configuration.host = 'https://git.curiosityworks.org/api/v1'
    return giteapy.UserApi(giteapy.ApiClient(configuration))


def get_repository_api_instance() -> giteapy.RepositoryApi:
    configuration = giteapy.Configuration()
    configuration.api_key['access_token'] = get_gitea_access_token()
    configuration.host = 'https://git.curiosityworks.org/api/v1'
    return giteapy.RepositoryApi(giteapy.ApiClient(configuration))


def get_repository_api_instance_user_cred(username: str, password: str) -> giteapy.RepositoryApi:
    configuration = giteapy.Configuration()
    configuration.username = username
    configuration.password = password
    configuration.host = 'https://git.curiosityworks.org/api/v1'
    return giteapy.RepositoryApi(giteapy.ApiClient(configuration))


def create_user(username: str, password: str, email: str):
    api_instance = get_api_instance()
    body = giteapy.CreateUserOption(email = email, password = password, username = username, must_change_password=False)
    try:
        # Create a user
        return api_instance.admin_create_user(body=body)
    except ApiException as e:
        print("Exception when calling AdminApi->admin_create_user: %s\n" % e)


def delete_user(username: str, debug: bool=False):
    api_instance = get_api_instance()
    try:
        # Create a user
        return api_instance.admin_delete_user(username)
    except ApiException as e:
        if debug:
            print("admin_delete_user: %s\n" % e)        
        print("deleting user failed")
    except Exception as e:
        print("deleting user failed")


def create_user_repo(username: str, repo_name: str, debug: bool=False):
    api_instance = get_api_instance()
    repository = giteapy.CreateRepoOption(auto_init=True, name=repo_name) # CreateRepoOption |

    try:
        # Create a user
        return api_instance.admin_create_repo(username, repository)
    except ApiException as e:
        if debug:
            print("Exception when calling create_user_repo: %s\n" % e)                
        print(f'failed to create user repo {repo_name}')


def delete_user_repo(username: str, repo_name: str, password: str, debug: bool=False):
    api_instance = get_repository_api_instance_user_cred(username, password)

    try:
        return api_instance.repo_delete(username, repo_name)
    except ApiException as e:
        if debug:
            print("Exception when calling create_user_repo: %s\n" % e)                
        print(f'failed to delete user repo {repo_name}')


def list_user_repos(username: str):
    api_instance = get_user_api_instance()

    try:
        return api_instance.user_current_list_repos(async_req=False)
    except ApiException as e:
        print("Exception when calling AdminApi->admin_create_user: %s\n" % e)                


def create_public_key(username: str, key_body: str):
    api_instance = get_api_instance()
    key = giteapy.CreateKeyOption(key=key_body, read_only=False, title=f'{username} public key') # CreateKeyOption |  (optional)

    try:
        # Create a user
        return api_instance.admin_create_public_key(username, key=key)
    except ApiException as e:
        print("Exception when calling AdminApi->admin_create_user: %s\n" % e)                        


import click
from vag import __version__
from vag.console.commands.vagrant import vagrant
from vag.console.commands.docker import docker
from vag.console.commands.remote import remote
from vag.console.commands.cx import cx
from vag.console.commands.gitea import gitea


@click.group()
def root():
    pass


@root.command()
def version():
    """Prints version"""
    print(__version__)


root.add_command(version)
root.add_command(vagrant)
root.add_command(docker)
root.add_command(remote)
root.add_command(cx)
root.add_command(gitea)


def main():
    root()


import configparser


def read(file_path, debug=False):
    data = {}
    envs = {}
    tags = []

    config = configparser.ConfigParser()
    config.read(file_path)
    section = config.sections()[0]
    for key in config[section]:
        val = config[section][key]

        if key.startswith('env.'):
            key = key.replace('env.', '')
            envs[key.upper()] = val
            continue

        if key.startswith('tag.'):
            # tag.1 = urlprefix-www.example.com/signin  
            # tag.2 = urlprefix-www.example.com/signup
            tags.append(val) 
            continue
        
        data[key] = val

    data['envs'] = envs
    data['tags'] = tags
    if debug:
        print(data)

    return data



import os
import sys
from sqlalchemy import Column, ForeignKey, Integer, String, DateTime
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.orm import Session
import yaml
from sqlalchemy import select
from vag.utils.cx_schema import *

def user_foo_test_data():
    engine = create_engine(get_connection_str())
    Base.metadata.bind = engine    
    DBSession = sessionmaker(bind=engine)
    session = DBSession()
    
    # ----------- meta data tables -----------
    vscode = IDE(name='vscode')
    intellij = IDE(name='intellij')
    pycharm = IDE(name='pycharm')
    goland = IDE(name='goland')
    session.add(vscode)
    session.add(intellij)
    session.add(pycharm)
    session.add(goland)
    session.commit()   


    ember_install = RuntimeInstall(name='emberjs', script_body="""# ember
sudo sudo npm install -g ember-cli
""")
    tmux_install = RuntimeInstall(name='tmux', script_body="""# tmux
sudo apt-get install -y tmux
echo -e "\n\nalias s='tmux new -A -s shared'" >> /home/coder/.zshrc
""")
    gh_cli_install = RuntimeInstall(name='github cli', script_body="""# github cli gh install
curl -fsSL https://cli.github.com/packages/githubcli-archive-keyring.gpg | sudo gpg --dearmor -o /usr/share/keyrings/githubcli-archive-keyring.gpg
echo "deb [arch=$(dpkg --print-architecture) signed-by=/usr/share/keyrings/githubcli-archive-keyring.gpg] https://cli.github.com/packages stable main" | sudo tee /etc/apt/sources.list.d/github-cli.list > /dev/null
sudo apt-get update
sudo apt-get install gh
""")
    poetry_install = RuntimeInstall(name='poetry', script_body="""# poetry
curl -sSL https://raw.githubusercontent.com/python-poetry/poetry/master/get-poetry.py | python3 -
echo -e "export PATH=\"\$HOME/.poetry/bin:\$PATH\"" >> ~/.zshrc
""")
    postgresql_install = RuntimeInstall(name='postgres', script_body="""# vag dependencies
sudo apt-get install -y postgresql
sudo apt-get install -y libpq-dev
""")
    session.add(ember_install)
    session.add(tmux_install)
    session.add(gh_cli_install)
    session.add(poetry_install)
    session.add(postgresql_install)
    session.commit()

    # ----------- user related instance tables -----------

    new_user = User(username='foo', email='foo@gmail.com', password='teachmecoding', private_key="""-----BEGIN RSA PRIVATE KEY-----
-----END RSA PRIVATE KEY-----""", public_key='rsa public key', google_id='1234')
    session.add(new_user)
    session.commit()

    user_ide_vscode = UserIDE(user=new_user, ide=vscode)
    user_ide_intellij = UserIDE(user=new_user, ide=intellij)
    user_ide_pycharm = UserIDE(user=new_user, ide=pycharm)
    user_ide_goland = UserIDE(user=new_user, ide=goland)
    session.add(user_ide_vscode)
    session.add(user_ide_intellij)
    session.add(user_ide_pycharm)
    session.add(user_ide_goland)
    session.commit()

    session.add(IDERuntimeInstall(user_ide=user_ide_vscode, runtime_install=ember_install))
    session.add(IDERuntimeInstall(user_ide=user_ide_vscode, runtime_install=tmux_install))
    session.add(IDERuntimeInstall(user_ide=user_ide_vscode, runtime_install=gh_cli_install))
    session.add(IDERuntimeInstall(user_ide=user_ide_vscode, runtime_install=poetry_install))
    session.add(IDERuntimeInstall(user_ide=user_ide_vscode, runtime_install=postgresql_install))
    session.commit()

    containers_repo = UserRepo(uri='git@github.com:foo/containers.git', user=new_user)
    vag_repo = UserRepo(uri='git@github.com:foo/vag.git', user=new_user)
    sites_repo = UserRepo(uri='git@github.com:foo/sites.git', user=new_user)
    users_repo = UserRepo(uri='git@github.com:foo/users.git', user=new_user)
    session.add(containers_repo)
    session.add(vag_repo)
    session.add(sites_repo)
    session.add(users_repo)
    session.commit()

    session.add(IDERepo(user_ide=user_ide_vscode, uri=containers_repo.uri))    
    session.add(IDERepo(user_ide=user_ide_vscode, uri=vag_repo.uri))    
    session.add(IDERepo(user_ide=user_ide_vscode, uri=sites_repo.uri))    
    session.add(IDERepo(user_ide=user_ide_vscode, uri=users_repo.uri))    
    session.commit()


def reset_test_data_foo(): 
    engine = create_engine(get_connection_str())
    Base.metadata.bind = engine
    Base.metadata.drop_all()
    Base.metadata.create_all(engine)
    user_foo_test_data()
    

if __name__ == '__main__':
    reset_test_data_foo()


import requests
import os
import sys

def get_ip_port(job_name: str, group: str, debug: bool):
    alloc = get_allocation(job_name, group, debug)

    ip = alloc['Resources']['Networks'][0]['IP']
    dynamic_ports = alloc['Resources']['Networks'][0]['DynamicPorts']
    port = ''
    for p in dynamic_ports:
        if p['Label'] == 'ssh':
            port = p['Value']
            break
    
    return ip, port


def get_version(job_name: str, group: str, debug: bool):
    if debug:
        print(f'job_name = {job_name}, group = {group}')
    job = get_job(job_name, debug)
    if job is None:
        return '1.0.0'

    task_groups = job['TaskGroups']
    for task_group in task_groups:
        task_group_name = task_group['Name']
        if task_group_name == group:
            image = task_group['Tasks'][0]['Config']['image']
            if debug:
                print(image)
            version = image[image.rfind(':')+1:]
            return version
    return ''


def get_job(job_name: str, debug: bool):
    nomad_addr = os.getenv('NOMAD_ADDR')
    if not nomad_addr:
        print('missing $NOMAD_ADDR environment variable')
        sys.exit(1)

    job_api_url = f'{nomad_addr}/v1/job/{job_name}'
    if debug:
        print(job_api_url)
    
    response = requests.get(job_api_url)
    if response:
        job = response.json()
        if debug:
            print(job)
        return job
    return None


def get_allocation(job_name: str, group: str, debug: bool):
    nomad_addr = os.getenv('NOMAD_ADDR')
    if not nomad_addr:
        print('missing $NOMAD_ADDR environment variable')
        sys.exit(1)

    allocations_api_url = f'{nomad_addr}/v1/job/{job_name}/allocations'
    if debug:
        print(allocations_api_url)
        
    allocations = requests.get(allocations_api_url).json()
    if debug:
        print(allocations)

    alloc_id = ''
    for a in allocations:
        task_group = a['TaskGroup']
        if debug:
            print(f'task_group = {task_group}')

        if not task_group == group:
            continue

        task_states = a['TaskStates']
        if 'container' in task_states: 
            task_state = task_states['container']['State']
            if task_state == 'running':
                alloc_id = a['ID']
        else:
            # for backwards compatibility, remove this after containers are redployed
            if f'{job_name}-service' in task_states:
                task_state = task_states['{job_name}-service']['State']
                if task_state == 'running':
                    alloc_id = a['ID']

    if debug:
        print(f'alloc_id = {alloc_id}')

    alloc = requests.get(f'{nomad_addr}/v1/allocation/{alloc_id}').json()
    if debug:
        print(alloc)    
    return alloc

# /Job/TaskGroups[0]/Tasks[0]/Config/image


import sys
from os.path import expanduser
import click
from vag.utils import exec
import untangle
from jinja2 import Template
from vag.utils import exec
from vag.utils import hash_util
import os
import sys
from os.path import expanduser


@click.group()
def vagrant():
    """ Vagrant Instance Automation """
    pass


@vagrant.command()
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def list(debug):
    """lists running vagrant instances""" 

    script_path = exec.get_script_path('instance/list.sh')

    returncode, lines = exec.run(script_path, True)
    if returncode != 0:
        sys.exit(1)
    
    home = expanduser("~")

    for instance in lines:
        if len(instance) > 0:
            f = open(f'{home}/VirtualBox VMs/{instance}/{instance}.vbox', 'r')
            xml_str = f.read()

            obj = untangle.parse(xml_str)
            machine = obj.VirtualBox.Machine
            os_type = machine['OSType']
            memory = machine.Hardware.Memory['RAMSize']
            name = machine['name']
            shared_folder = machine.Hardware.SharedFolders.SharedFolder['hostPath']
            vagrant_f = open(shared_folder+'/Vagrantfile', 'r')
            ip = ''
            for line in vagrant_f.readlines():
                if 'config.vm.network' in line and 'ip:' in line:
                    tokens = line.split(',')
                    for token in tokens:
                        if 'ip:' in token:
                            ip = token.replace('ip:', '').strip().replace('"', '')          

            print(f"""
name   : {name}
shared : {shared_folder}
os     : {os_type}
ip     : {ip}
memory : {memory}""")
            f.close()


@vagrant.command()
@click.argument('box', default='7onetella/ubuntu-20.04', metavar='<box>')
@click.option('--hostname', default='', metavar='<hostname>')
@click.option('--ip_address', default='', metavar='<ip_address>')
@click.option('--interface', default='', help='network interface')
@click.option('--memory', default='512', help='memory')
@click.option('--service', default='', help='service to start')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def init(box, hostname, ip_address, interface, memory, service, debug):
    """Creates a new Vagrantfile"""

    home = expanduser("~")

    # if not hostname:
    #   cwd = os.getcwd()
    #   current_folder_name = cwd[cwd.rfind('/')+1:]
    #   hostname = current_folder_name

    # config.vm.box_url = "file://{{ home }}/.vagrant/boxes/{{ box }}/package.box"

    template = Template("""
Vagrant.configure("2") do |config|

  config.vm.box = "{{ box }}"{% if ip_address|length %}
  config.vm.network "public_network", ip: "{{ ip_address }}", bridge: "{{ interface }}"
  {% endif %}{% if service|length %}
  config.vm.provision "shell",
    run: "always",
    inline: "sleep 60; systemctl start {{ service }}"
  {% endif %}{% if hostname|length %}
  config.vm.provider "virtualbox" do |vb|
    vb.name   = "{{ hostname }}"{% if memory|length %}
    vb.memory = "{{ memory }}"{% endif %}
  end
  
  config.vm.hostname          = "{{ hostname }}"{% endif %}

  config.ssh.insert_key       = false
  config.ssh.private_key_path = ['~/.vagrant.d/insecure_private_key', '~/.ssh/id_rsa']
  config.vm.provision "file", source: "~/.ssh/id_rsa.pub", destination: "~/.ssh/authorized_keys"

end""")

    output = template.render(
        box=box,
        home=home,
        hostname=hostname,
        memory=memory,
        ip_address=ip_address,
        interface=interface,
        service=service
    )
    f = open('./Vagrantfile', 'w+')
    f.write(output)
    f.close()

    if debug:
        print(output)


@vagrant.command()
@click.argument('box', default='', metavar='<box>')
@click.option('--base', default='7onetella/ubuntu-20.04', metavar='<base>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def build(box, base, debug):
    """Builds vagrant box"""

    organization = box[:box.rfind('/')]
    box_name = box[box.rfind('/')+1:box.rfind(':')]
    version_ = box[box.rfind(':') + 1:]

    template = Template("""
    {
      "builders": [
        {
          "box_name"    : "{{ box }}",
          "output_dir"  : "/tmp/vagrant/build/{{ organization }}/{{ box_name }}",
          "box_version" : "{{ version }}",      
          "communicator": "ssh",
          "source_path" : "{{ base }}",
          "provider"    : "virtualbox",
          "skip_add"    : true,
          "type"        : "vagrant"
        }
      ],
      "provisioners": [
        {
          "ansible_env_vars": [ "ANSIBLE_STDOUT_CALLBACK=debug" ],
          "extra_arguments" : [ "--extra-vars", "target=default user=vagrant ansible_os_family=Debian" ],
          "type"            : "ansible",
          "playbook_file"   : "{{ box_name }}.yml",
          "user"            : "vagrant"
        }
      ]       
    }""")

    try:
        os.makedirs(f'/tmp/vagrant/template/{organization}/{box_name}')
    except OSError:
        # do nothing
        pass

    output = template.render(
        box=box,
        base=base,
        box_name=box_name,
        organization=organization,
        version=version_
    )
    template_path = f'/tmp/vagrant/template/{organization}/{box_name}/{box_name}.json'
    f = open(template_path, 'w+')
    f.write(output)
    f.close()
    if debug:
        print(output)

    script_path = exec.get_script_path(f'build/box.sh clean {organization} {box_name}')
    returncode, lines = exec.run(script_path, False)
    if returncode != 0:
        sys.exit(1)

    script_path = exec.get_script_path(f'build/box.sh build {template_path}')
    returncode, lines = exec.run(script_path, False)
    if returncode != 0:
        sys.exit(1)


@vagrant.command()
@click.argument('box', default='', metavar='<box>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
@click.option('--skip', is_flag=True, default=False, help='skip copying the box')
def push(box, debug, skip):
    """Publishes vagrant box to target environment"""

    organization = box[:box.rfind('/')]
    box_name = box[box.rfind('/')+1:box.rfind(':')]
    version_ = box[box.rfind(':') + 1:]

    if not skip:
        print("scp-ing the box")
        script_path = exec.get_script_path(f'build/box.sh push {organization} {box_name}')
        returncode, lines = exec.run(script_path, False)
        if returncode != 0:
            sys.exit(1)

    try:
        os.makedirs(f'/tmp/vagrant/metadata/{organization}/{box_name}')
    except OSError:
        # do nothing
        pass

    metadata_template = Template("""
    {
      "description": "",
       "name": "{{ organization }}/{{ box_name }}",
       "versions": [
         {
           "providers": [
             {
              "checksum": "{{ sha1sum }}",
              "checksum_type": "sha1",
              "name": "virtualbox",
              "url": "http://tmt.7onetella.net/boxes/{{ organization }}/{{ box_name }}/package.box"
             }
           ],
           "version": "{{ version }}"
         }
       ]
    }""")
    metadata_output = metadata_template.render(
        box=box,
        organization=organization,
        box_name=box_name,
        sha1sum=hash_util.sha1sum(f'/tmp/vagrant/build/{organization}/{box_name}/package.box'),
        version=version_
    )
    metadata_json_path = f'/tmp/vagrant/metadata/{organization}/{box_name}/metadata.json'
    f = open(metadata_json_path, 'w+')
    f.write(metadata_output)
    f.close()
    if debug:
        print(metadata_output)

    print("scp-ing the metadata.json")
    script_path = exec.get_script_path(f'build/box.sh metadata {organization} {box_name}')
    returncode, lines = exec.run(script_path, False)
    if returncode != 0:
        sys.exit(1)


@vagrant.command()
@click.argument('box', default='', metavar='<box>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def test(box, debug):
    """Start a test Vagrant instance"""

    organization = box[:box.rfind('/')]
    box_name = box[box.rfind('/')+1:]

    script_path = exec.get_script_path(f'build/box.sh test {organization} {box_name}')
    returncode, lines = exec.run(script_path, False)
    if returncode != 0:
        sys.exit(1)


@vagrant.command()
@click.argument('box', default='', metavar='<box>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def ssh(box, debug):
    """SSH to vagrant test Vagrant instance"""

    organization = box[:box.rfind('/')]
    box_name = box[box.rfind('/')+1:]

    script_path = exec.get_script_path(f'build/box.sh ssh {organization} {box_name}')
    exec.fork(script_path, debug)


@vagrant.command()
@click.argument('box', default='', metavar='<box>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def clean(box, debug):
    """Cleans up vagrant build, terminates vagrant instance etc"""

    organization = box[:box.rfind('/')]
    box_name = box[box.rfind('/')+1:]

    script_path = exec.get_script_path(f'build/box.sh clean {organization} {box_name}')
    returncode, lines = exec.run(script_path, False)
    if returncode != 0:
        sys.exit(1)




import os
import sys
from sqlalchemy import Column, ForeignKey, Integer, String, DateTime
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.orm import Session
import yaml
from sqlalchemy import select
from vag.utils.cx_schema import *
import time


def init_session_future():
    conn_str = get_connection_str()
    if not conn_str:
        return None
    engine = create_engine(conn_str, isolation_level="AUTOCOMMIT")
    return Session(engine)


db_session = init_session_future()


def get_session(): 
    conn_str = get_connection_str()
    if not conn_str:
        print('CX_DB_CONN must be set')
        sys.exit(1)

    return db_session


def find_one(statement):
    try:
        return get_session().execute(statement).scalars().one()    
    except:
        return None


def find_user_by_google_id(google_id: str) -> User:
    statement = select(User).filter_by(google_id=google_id)
    return find_one(statement) 


def find_enrollment_by_hashed_email(hashed_email: str) -> Enrollment:
    statement = select(Enrollment).filter_by(hashed_email=hashed_email)
    return find_one(statement)     


def find_user_by_username(username: str) -> User:
    statement = select(User).filter_by(username=username)
    return get_session().execute(statement).scalars().one()


def find_runtime_install_by_name(runtime_install_name: str):
    statement = select(RuntimeInstall).filter_by(name=runtime_install_name)
    return get_session().execute(statement).scalars().one()


def find_ide_runtime_installs_by_user_id(user_ide_id):
    statement = select(IDERuntimeInstall).filter_by(user_ide_id=user_ide_id)
    return get_session().execute(statement).scalars().all()   


def find_user_ides_by_user_id(user_id):
    statement = select(UserIDE).filter_by(user_id=user_id)
    return get_session().execute(statement).scalars().all()   


def find_user_ide_by_user_id_ide_name(user_id, ide_name: str):
    user_ides = find_user_ides_by_user_id(user_id)
    for user_ide in user_ides:
        if user_ide.ide.name == ide_name:
            return user_ide

def find_user_repos_by_user_id(user_id):
    statement = select(UserRepo).filter_by(user_id=user_id)
    return get_session().execute(statement).scalars().all()   


def find_ide_repos_by_user_ide_id(user_ide_id):
    statement = select(IDERepo).filter_by(user_ide_id=user_ide_id)
    return get_session().execute(statement).scalars().all()   


def find_ide_by_name(ide_name: str):
    statement = select(IDE).filter_by(name=ide_name)
    return get_session().execute(statement).scalars().one()


def get_build_profile(username: str, ide_name: str) -> dict:
    user = find_user_by_username(username)

    user_ide = find_user_ide_by_user_id_ide_name(user.id, ide_name)

    ide_repos = find_ide_repos_by_user_ide_id(user_ide.id)

    statement = select(IDERuntimeInstall).filter_by(user_ide=user_ide)
    user_ide_runtime_installs = get_session().execute(statement).scalars().all()

    bodies = [ u_r_i.runtime_install.script_body for u_r_i in user_ide_runtime_installs ]
    snppiets = []
    for body in bodies:
        snppiets.append({'body': body})

    return {
        'ide': ide_name,
        'username': username,
        'password': user.password,
        'email': user.email,
        'private_key': user.private_key,
        'public_key': user.public_key,
        'repositories': [repo.uri for repo in ide_repos],
        'snippets': snppiets
    }




import os
import sys
import shlex
import click
import requests
from os.path import expanduser
from vag.utils import exec
from vag.utils import config
from vag.utils.misc import create_ssh

@click.group()
def remote():
    """ Remote builder automation """
    pass


@remote.command()
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def init(debug: bool):
    """initializes go related config"""

    home = expanduser("~")
    vag_config = f'{home}/.vag.conf'

    if os.path.isfile(vag_config):
        print('config file exists')

    # data = config.read(vag_config)


@remote.command()
@click.argument('repo', default='', metavar='<repo>')
@click.argument('branch', default='', metavar='<branch>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def build(repo: str, branch: str, debug: bool):
    """builds your project"""
    script_path = exec.get_script_path(f'go.sh build {repo} {branch}')

    if debug:
        print(f'script_path={script_path}')
    return_code = exec.run_raw(script_path)
    if return_code != 0:
        sys.exit(1)


@remote.command()
@click.argument('repo', default='', metavar='<repo>')
@click.argument('stage', default='', metavar='<stage>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def deploy(repo: str, stage: str, debug: bool):
    """deploys your project"""
    script_path = exec.get_script_path(f'go.sh deploy {repo} {stage}')
    return_code = exec.run_raw(script_path)
    if return_code != 0:
        sys.exit(1)


import os
import click
import sys
from jinja2 import Template
from vag.utils import config
from vag.utils import exec
from vag.utils.nomadutil import get_version, get_ip_port
from vag.utils.misc import create_ssh, do_scp
from vag.utils.string_util import get_service_and_group
import yaml
import stat


@click.group()
def docker():
    """ Docker automation """
    pass


@docker.command()
@click.argument('semver', default='', metavar='<major|minor|patch>')
@click.argument('name', default='', metavar='<service>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def version(semver: str, name:str, debug: bool):
    """calculate next release version using semver"""

    # password-dev or codeserver-f121-public
    service, group = get_service_and_group(name)
    current_version = get_version(service, group, debug)

    major = int(current_version.split('.')[0])
    minor = int(current_version.split('.')[1])
    patch = int(current_version.split('.')[2])

    if semver == 'major':
        next_major = major + 1
        print(f'{next_major}.{minor}.{patch}')
        return

    if semver == 'minor':
        next_minor = minor + 1
        print(f'{major}.{next_minor}.{patch}')
        return

    if semver == 'patch':
        next_patch = patch + 1
        print(f'{major}.{minor}.{next_patch}')
        return


@docker.command()
@click.argument('repo_name_revision', default='', metavar='<service>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def deploy(repo_name_revision, debug):
    """deploys docker image in nomad environment"""

    # docker-registry.7onetella.net/7onetella/password-dev:0.8.4
    last_slash_idx = repo_name_revision.rfind('/')
    docker_registry_uri = repo_name_revision[:last_slash_idx]
    name_revision = repo_name_revision[last_slash_idx+1:]

    tokens = name_revision.split(':')
    service, group = get_service_and_group(tokens[0])
    if debug:
        print(f'service = {service}, group = {group}')
    version = tokens[1]

    image = f'{docker_registry_uri}/{service}:{version}'
    if debug:
        print(f'image = {image}')

    template = Template("""
    job "{{ service }}" {
      datacenters = ["dc1"]

      type = "service"

      update {
        stagger      = "60s"
        max_parallel = 1
      }

      group "{{ group }}" {
        count = 1
        network {
            port "http" { to = {{ port }} }
            port "ssh"  { to = 22 }
        }            
            
        task "container" {
            driver = "docker"
            config {
                image = "{{ image }}"
                ports = [ "http", "ssh" ]{% if log_driver is not none %}
                
                logging {
                   type = "elasticsearch"
                   config {
                        elasticsearch-url="https://elasticsearch-dev.7onetella.net:443"
                        elasticsearch-sniff=false
                        elasticsearch-index="docker-%F"
                        elasticsearch-type="_doc"
                        elasticsearch-timeout="60s"
                        elasticsearch-version=5
                        elasticsearch-fields="containerID,containerName,containerImageName"
                        elasticsearch-bulk-workers=1
                        elasticsearch-bulk-actions=1000
                        elasticsearch-bulk-size=1024
                        elasticsearch-bulk-flush-interval="1s"                   
                    }
                }{% endif %}

                volumes = [
                    "/var/run/docker.sock:/var/run/docker.sock"
                ]                
            }
    
            resources {
                cpu = 20
                memory = {{ memory }}
            }{% if health_check is not none %}

            service {
                tags = [ {% for tag in tags %}{% if loop.index0 > 0 %},{% endif %} "{{tag}}"{% endfor %} ]
                port = "http"
                check {
                    type     = "http"
                    path     = "{{ health_check }}"
                    interval = "10s"
                    timeout  = "2s"
                }
            }{% else %}

            service {
                port = "ssh"
            }{% endif %}
    
            env {  {% for key, value in envs.items() %}
                {{ key }} = "{{ value }}"{% endfor %}                
            }
        }
      }
    }""")

    current_dir = os.getcwd()
    app_file = f'{current_dir}/{service}-{group}.app'
    if debug:
        print(f'app_file = {app_file}')
        
    try:
        data = config.read(app_file)
    except IndexError:
        print(f'error while processing {app_file}')
        sys.exit(1)

    if debug:
        print(f'data is \n {data}')

    # if image is specified use it stead of deriving it from service name
    image_from_config = get(data, 'image', '')
    if image_from_config:
        image = image_from_config

    tags = data['tags']

    if not tags:        
        urlprefix = f'urlprefix-{ service }-{ group }.7onetella.net/'

        host = get(data, 'host', '')
        path = get(data, 'path', '/')
        if host:
            urlprefix = f'urlprefix-{host}{path}'

        tags.append(urlprefix)
    if debug:
        print(f'tags = {tags}')

    try:
        os.makedirs(f'/tmp/nomad')
    except OSError:
        # do nothing
        pass

    output = template.render(
        service=service,
        group=group,
        image=image,
        memory=get(data, 'memory', 128),
        port=get(data, 'port', 4242),
        health_check=get(data, 'health', None),
        log_driver=get(data, 'log_driver', None),
        tags=tags,
        envs=data['envs']
    )
    template_path = f'/tmp/nomad/{service}-{group}.nomad'
    f = open(template_path, 'w+')
    f.write(output)
    f.close()
    if debug:
        print(output)
        return 0

    script_path = exec.get_script_path(f'nomad.sh {template_path}')
    returncode, lines = exec.run(script_path, False)
    if returncode != 0:
        sys.exit(1)


@docker.command()
@click.argument('name', default='', metavar='<service>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def ssh(name:str, debug: bool):
    """SSH into docker container"""
    service, group = get_service_and_group(name)

    ip, port = get_ip_port(service, group, debug)
    if debug:
        print(f'ip = {ip}, port = {port}')
    
    landing_path = '/home/coder'
    if 'codeserver' in service:
        landing_path = '/home/coder/workspace'

    create_ssh(ip, port, 'coder', debug, landing_path, 'zsh')


@docker.command()
@click.argument('name', default='', metavar='<service>')
@click.argument('src', default='', metavar='<src>')
@click.argument('target', default='', metavar='<target>')
@click.option('--show', is_flag=True, default=False, help='print scp command')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def scp(name:str, src: str, target: str, show: bool, debug: bool):
    """SCP to docker container"""
    service, group = get_service_and_group(name)

    ip, port = get_ip_port(service, group, debug)
    if debug:
        print(f'ip = {ip}, port = {port}')

    do_scp(ip, port, 'coder', src, target, show, debug)


@docker.command()
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def pre_build(debug: bool):
    """Generates config files need by docker build"""

    document = ""
    # if tty
    # if sys.stdin.isatty():
    #     document = read_file(f'./profile-{username}.yml')
    # else: # if pipe
    for line in sys.stdin:
        document += line

    profile = yaml.load(document, Loader=yaml.FullLoader)
    if debug: 
        print(f'profile = {profile}')

    username = profile['username']
    password = profile['password']
    email = profile['email']

    # ------------------------------------------------------------------
    if 'private_key' in profile:
        try:
            os.makedirs(f'./.ssh', mode=0o700)
        except OSError:
            # do nothing
            pass
        if profile['private_key']:
            write_file('./.ssh/id_rsa', profile['private_key'])
            st = os.stat('./.ssh/id_rsa')
            os.chmod('./.ssh/id_rsa', stat.S_IRUSR | stat.S_IWUSR)

    # ------------------------------------------------------------------
    write_file('./config.yml', render_template("""bind-addr: 0.0.0.0:9991
auth: password
password: {{ password }}
cert: false    
""", password=password))

    # ------------------------------------------------------------------
    write_file('./.gitconfig', render_template("""[credential]
        helper = store
[user]
	name = {{ username }}
	email = {{ email }}""", username=username, email=email))

    # ------------------------------------------------------------------
    app_file_path = f'./{profile["ide"]}-{username}-public.app'
    write_file(app_file_path, render_template("""[vscode]
memory  = 2048
port    = 9991
health  = /
host    = {{ ide }}-{{ username }}.curiosityworks.org
""", username=username, ide=profile['ide']))

    # ------------------------------------------------------------------
    if 'repositories' in profile and profile['repositories']:
        write_file('./repositories.txt', render_template("""{% for repo_uri in repositories %}{% if loop.index0 > 0 %}\n{% endif %}{{ repo_uri }}{% endfor %}""", repositories=profile['repositories']))
    else:
        write_file('./repositories.txt', '')
        
    # ------------------------------------------------------------------
    if 'snippets' in profile:
        snippets = [snippet['body'] for snippet in profile['snippets']]    
        write_file('./runtime_install.sh', render_template("""#!/bin/bash -e
    
set -x

{% for snippet in snippets %}{{ snippet }}

{% endfor %}# snippets end here""", snippets=snippets))
        st = os.stat('./runtime_install.sh')
        os.chmod('./runtime_install.sh', st.st_mode | stat.S_IEXEC)
    # ------------------------------------------------------------------
    if True:
        write_file('./gotty.sh', render_template("""#!/bin/sh

export TERM=xterm

/home/coder/go/bin/gotty --ws-origin "vscode-{{ username }}.curiosityworks.org" -p 2222 -c {{ username }}:{{ password }} -w /usr/bin/zsh >>/dev/null 2>&1 
""", username=username, password=password))
        st = os.stat('./gotty.sh')
        os.chmod('./gotty.sh', st.st_mode | stat.S_IEXEC)


@docker.command()
@click.argument('service', default='', metavar='<service>')
@click.option('--debug', is_flag=True, default=False, help='debug this command')
def post_build(service: str, debug: bool):
    """Deletes config files generated in pre-build"""

    delete_file('./.ssh/id_rsa')

    delete_file('./config.yml')

    delete_file('./.gitconfig')

    delete_file(f'{service}-public.app')

    delete_file('./repositories.txt')

    delete_file('./runtime_install.sh')

    delete_file('./gotty.sh')


def get(data: dict, key: str, default_value):
    if key in data:
        return data[key]
    else:
        if default_value:
            return default_value
        else:
            return None


def write_file(file_path: str, body: str):
    print(f'writing {file_path}')
    f = open(file_path, 'w+')
    f.write(body)
    f.close()


def read_file(file_path: str) -> str:
    print(f'reading {file_path}')
    f = open(file_path, 'r')
    body = f.read()
    f.close()
    return body


def delete_file(file_path: str):
    if os.path.exists(file_path):
        print(f'deleting {file_path}')
        os.remove(file_path)


def render_template(tpl_body: str, **kwargs):
    template = Template(tpl_body)
    output = template.render(
        kwargs
    )
    return output




